#include <ArduinoJson.h>
#include <rtl_433_ESP.h>
#include <RCSwitch.h>
#include "Dictionary.h"


#define _DICT_PACK_STRUCTURES

#define LED 8

CC1101 radiotx = RADIO_LIB_MODULE;
RCSwitch mySwitch = RCSwitch();

#ifndef RF_MODULE_FREQUENCY
#  define RF_MODULE_FREQUENCY 433.92
#endif

#define JSON_MSG_BUFFER 512
#define MINDELAY 10L

Dictionary<String, unsigned long> d1;

char messageBuffer[JSON_MSG_BUFFER];

rtl_433_ESP rf; // use -1 to disable transmitter

int count = 0;

void rtl_433_Callback(char* message) {
  JsonDocument jsonDocument;
  deserializeJson(jsonDocument,message);
  logJson(jsonDocument);
  count++;
}

void logJson(JsonDocument jsondata) {
  if(jsondata["model"].is<String>())
  {
    String ref = jsondata["model"];
    if(jsondata["channel"].is<int>()) { ref = ref + jsondata["channel"].as<String>(); }
    if(jsondata["id"].is<int>()) { ref = ref + jsondata["id"].as<String>(); }
    if(!d1.contains(ref.c_str()) || millis() > (d1.get(ref.c_str()) + (MINDELAY * 1000)) || d1.get(ref.c_str()) > millis()) {
      //Serial.println(ref.c_str());
      //if(d1.contains(ref.c_str())) { Serial.println(d1.get(ref.c_str())); }
      #if defined(ESP8266) || defined(ESP32) || defined(__AVR_ATmega2560__) || defined(__AVR_ATmega1280__)
      char JSONmessageBuffer[measureJson(jsondata) + 1];
      serializeJson(jsondata, JSONmessageBuffer, measureJson(jsondata) + 1);
      #else
      char JSONmessageBuffer[JSON_MSG_BUFFER];
      serializeJson(jsondata, JSONmessageBuffer, JSON_MSG_BUFFER);
      #endif
      d1.set(ref.c_str(), millis());
      ledblink();
      Serial.println(JSONmessageBuffer);
    }
  }
}

void ledblink() {
  digitalWrite(LED, LOW);
  delay(150);
  digitalWrite(LED, HIGH);
}

void setup() {
  Serial.begin(115200);
  delay(1000);
  for (int i=0 ; i<10; i++) {
       Serial.print("Hello");
      delay(1000);
  }
  if(SS != 21 || MOSI != 20 || MISO != 10 || SCK != 7)
  {
    for ( ; ; ) {
      Serial.println("Please define pin assignment for SPI in pins_arduino.h (~/.platformio/packages/framework-arduinoespressif32/variants/esp32c3) / SS: 21 MOSI : 20 MISO : 10 SCK : 7");
      delay(1000);
    }
  }
  pinMode(LED, OUTPUT);
  digitalWrite(LED, HIGH);
  rf.initReceiver(RF_MODULE_RECEIVER_GPIO, RF_MODULE_FREQUENCY);
  rf.setCallback(rtl_433_Callback, messageBuffer, JSON_MSG_BUFFER);
  rf.enableReceiver();
  rf.getModuleStatus();

  //d = new Dictionary(25);
  //d("aa", "bb");
  //if(d1("aa")) { Serial.println("toto"); }
  
}

void loop() {
  rf.loop();
  if (Serial.available()) 
  {
    // Read the JSON document from the "link" serial port
    JsonDocument doc;
    DeserializationError err = deserializeJson(doc, Serial);

    if (err == DeserializationError::Ok) 
    {
       Serial.print("cmd = ");
       Serial.println(doc["cmd"].as<String>());
       Serial.print("value = ");
       Serial.println(doc["value"].as<uint32_t>());
       if(doc["cmd"].as<String>().compareTo(String("rcsend"))==0) {
         Serial.println("Transmit RCSwitch");
         ledblink();
	 //rf.getModuleStatus();
	 rf.disableReceiver();
	 radiotx.SPIsendCommand(RADIOLIB_CC1101_CMD_TX);
         mySwitch.enableTransmit(RF_MODULE_GDO0);
         mySwitch.setRepeatTransmit(8);
         mySwitch.send(doc["value"].as<uint32_t>(), 24);
         mySwitch.disableTransmit();
	 radiotx.SPIsendCommand(RADIOLIB_CC1101_CMD_RX);
	 rf.enableReceiver();
	 //rf.getModuleStatus();
       }
    } 
    else 
    {
      // Print error to the "debug" serial port
      Serial.print("deserializeJson() returned ");
      Serial.println(err.c_str());
  
      // Flush all bytes in the "link" serial port buffer
      while (Serial.available() > 0)
        Serial.read();
    }
  }
}
